package jp.ikedam.jenkins.plugins.copyartifactwithslash;

import hudson.EnvVars;
import hudson.Extension;
import hudson.model.Descriptor;
import hudson.model.Job;
import hudson.model.Run;
import hudson.plugins.copyartifact.BuildSelector;
import hudson.plugins.copyartifact.StatusBuildSelector;
import hudson.plugins.copyartifact.BuildFilter;
import hudson.plugins.copyartifact.ParametersBuildFilter;
import hudson.plugins.copyartifact.SimpleBuildSelectorDescriptor;
import org.kohsuke.stapler.DataBoundConstructor;

/**
 * CopyArtifactでスラッシュが使えない問題を回避するためのBuildSelector。
 * 
 * 通常のCopyArtifactの処理は以下のようになっている。
 * 1. CopyArtifact::perform で、プロジェクト名を変数解決する
 * 2. CopyArtifact::JobResolver で、ジョブを特定する。
 *    スラッシュがある場合、スラッシュより後ろ(ビルドパラメータの条件)を
 *    ParametersBuildFilter として作成する。
 *      → このとき、スラッシュの位置を lastIndexOf()で取得しているのが全ての問題の発端。
 * 3. 設定で指定されたBuildSelector(通常、「最新の成功ビルド」の場合、StatusBuildSelector)で
 *    ビルドを取得する。
 * 
 * lastIndexOfの問題を回避するため、以下のように処理をする。
 * ビルドパラメータの指定を別のパラメータとして受け取るようにする。
 * 指定されたパラメータを変数解決し、ParametersBuildFilterを作成する。
 * 上位から与えられたfilterを無視して、自分で作ったfilterをStatusBuildSelectorに引き渡す。
 * 
 * @see BuildSelectorParameter
 * @author IKEDAM <devld@ikedam.jp>
 */
public class SlashBuildSelector extends StatusBuildSelector
{
    /**
     * ビルドパラメータの指定。
     * param1=value1,param2=value2, の形式で受け取る。
     */
    private String buildParameter;
    
    /**
     * 画面表示とかで使うgetter。
     * @return 設定されたビルドパラメータ
     */
    public String getBuildParameter(){
        return buildParameter;
    }
    
    /**
     * 設定(config.xml)から復元される時に呼び出される。
     * @param buildParameter 設定されたビルドパラメータの指定。
     */
    @DataBoundConstructor
    public SlashBuildSelector(String buildParameter, boolean stableOnly)
    {
        super(stableOnly);
        this.buildParameter = buildParameter;
    }
    
    /**
     * コピー元のビルドを取得する処理。
     * @param job コピー元のジョブ
     * @param env 環境変数
     * @param filter ビルドの条件フィルタ。無視します。
     * @param parent コピー先のジョブ
     * 
     * @return コピー元のビルド
     */
    @Override
    public Run<?,?> getBuild(Job<?,?> job, EnvVars env, BuildFilter filter, Run<?,?> parent)
    {
        // 変数展開したビルドパラメータ条件を取得
        String expandedBuildParameter = env.expand(buildParameter);
        
        // パラメータからフィルタを作成
        ParametersBuildFilter parameterFilter = new ParametersBuildFilter(expandedBuildParameter);
        
        if(!parameterFilter.isValid(job))
        {
            // パラメータがジョブで有効なものでないとかそういうい状況
            // TODO: ログを吐く
            return null;
        }
        
        return super.getBuild(job, env, parameterFilter, parent);
    }
    
    /**
     * プラグインの情報。
     * この情報によって、resources 以下のファイル(jellyとかpropertiesとか)が参照されるようになる。のだと思う。
     */
    @Extension
    public static final Descriptor<BuildSelector> SlashBuildSelectorDescriptor =
        new SimpleBuildSelectorDescriptor(
            SlashBuildSelector.class,
            Messages._SlashBuildSelector_DisplayName()
        );
}
